#include <QApplication>
#include <QMainWindow>
#include <QButtonGroup>
#include <QVBoxLayout>
#include <QStackedWidget>
#include <QGroupBox>
#include <QStyle>
#include <QLabel>
#include <QPaintEvent>
#include <QDebug>
#include <QWebEngineView>
#include <QAbstractButton>

class MyButton : public QAbstractButton
{
    Q_OBJECT
public:
    explicit MyButton(QWidget *parent = 0) : QAbstractButton(parent)
    { setCheckable(true); }

protected:
    void paintEvent(QPaintEvent *event) Q_DECL_OVERRIDE;
};

void MyButton::paintEvent(QPaintEvent *event)
{
    static int n = 0;
    ++n;
    qDebug() << __FUNCTION__ << this << isChecked() << isDown() << n;
    QPainter p(this);

    if(isDown() || isChecked()) {
        p.setPen(Qt::NoPen);
        p.setBrush(QColor("#428bca"));
        p.drawRect(event->rect());
        p.setPen(Qt::white);
    } else {
        p.setPen(QColor("#428bca"));
    }
    p.drawText(6, 28, text() + QLatin1String(" paintEvent #") + QString::number(n));
    p.end();
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);

private slots:
    void button1Clicked()  { m_stack->setCurrentIndex(0); qDebug() << __FUNCTION__; }
    void button3Clicked()  { m_stack->setCurrentIndex(2); qDebug() << __FUNCTION__; }

private:
    QStackedWidget *m_stack;
};

MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent)
{
    QWidget *cw = new QWidget;
    QVBoxLayout *layout = new QVBoxLayout(cw);
    setCentralWidget(cw);

    QButtonGroup *group = new QButtonGroup(this);

    MyButton *button1 = new MyButton(cw);
    MyButton *button2 = new MyButton(cw);
    MyButton *button3 = new MyButton(cw);

    layout->addWidget(button1);
    layout->addWidget(button2);
    layout->addWidget(button3);

    group->addButton(button1);
    group->addButton(button2);
    group->addButton(button3);

    button1->setText("Button 1");
    button1->setObjectName("button1");
    button2->setText("Button 2");
    button2->setObjectName("button2");
    button3->setText("Button 3");
    button3->setObjectName("button3");

    button2->setEnabled(false);

    button1->setMinimumSize(50,50);
    button2->setMinimumSize(50,50);
    button3->setMinimumSize(50,50);

    m_stack = new QStackedWidget;
    layout->addWidget(m_stack);
    QWebEngineView *wv = new QWebEngineView;
    wv->setUrl(QUrl::fromUserInput("https://bugreports.qt.io/browse/QTBUG-52123"));
    m_stack->addWidget(wv);
    QLabel *lbl = new QLabel("Stack page #2");
    lbl->setFrameShape(QFrame::Box);
    m_stack->addWidget(lbl);
    lbl = new QLabel("Stack page #3");
    lbl->setFrameShape(QFrame::Box);
    m_stack->addWidget(lbl);

    connect(button1, &MyButton::clicked, this,&MainWindow::button1Clicked);
    connect(button3, &MyButton::clicked, this,&MainWindow::button3Clicked);
}

int main(int argc, char *argv[])
{
	QApplication a(argc, argv);
	MainWindow w;
    QString t = QLatin1String(QT_VERSION_STR) + QLatin1Char(' ') + QGuiApplication::platformName()
            + QLatin1Char(' ') + a.style()->objectName();
    qDebug() << t;
    w.setWindowTitle(t);
	w.show();

	return a.exec();
}

#include "main.moc"
